/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */
package jorm.rdb;

// Import clauses required by jorm
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PAccessorGen;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Date;

// Import clauses required by the object Cmp2_CreditCard
import jorm.Cmp2_CreditCardAccessor;
// Imports needed by the rdb mapper
import org.objectweb.transaction.api.ConnectionWrapper;
import java.sql.Blob;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.util.HashMap;

public class Cmp2_CreditCardBinding
	extends org.objectweb.jonas_ejb.container.JEntitySwitch
	implements PBindingCtrl
 
 {
	/**
	 * The current state of this PBinding object.
	 */
	protected byte status = PBinding.LIFECYCLE_NOTBOUND;

	/**
	 * The PClassMapping linked to this PBinding object.
	 */
	protected Cmp2_CreditCardMapping pClassMapping = null;

	/**
	 * The PName of this PBinding object.
	 */
	protected PName pname = null;

	// Declare the Connection and the ResultSet used (java.sql)
	private Connection connection = null;
	private ResultSet resultSet = null;

	public Cmp2_CreditCardBinding() {
		super();
// nothing
	}

	public Cmp2_CreditCardBinding(PClassMapping pcm) {
		super();
		pClassMapping = (Cmp2_CreditCardMapping) pcm;
// nothing
	}


	// IMPLEMENTATION OF METHODS FROM THE PBinding INTERFACE

	public void init(PClassMapping pcm) throws PException {
		pClassMapping = (Cmp2_CreditCardMapping) pcm;
	}

	public synchronized void write(Object conn, PAccessor pa)
		throws PException {
		Cmp2_CreditCardAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof Cmp2_CreditCardAccessor)
			paccessor = (Cmp2_CreditCardAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor");
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_WRITE);
		if (nextState == PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized operation");
		if (conn==null)
			throw new PExceptionProtocol("Impossible to write without connection");
		else if (conn instanceof Connection)
			connection = (Connection) conn;
		else if (conn instanceof ConnectionWrapper)
			connection = (Connection) ((ConnectionWrapper) conn).getNonCCIConnection();
		else
			throw new PExceptionProtocol("Impossible to write with this connection type.");
		try {
			Blob blob;
		if (status == PBinding.LIFECYCLE_DELTOWRITE) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Remove");
            PreparedStatement pstmt = connection.prepareStatement("DELETE FROM CMP2_CREDITCARD_ WHERE id_ = ? ");
 
            java.lang.Integer id_ = pname.encodeOint();
            if (id_==null) {
                pstmt.setNull(1, 4);
            } else {
                pstmt.setObject(1, id_);
            }
            pstmt.executeUpdate();
			pstmt.close();
        }
		else if (status==PBinding.LIFECYCLE_NEWTOWRITE) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Insert");
            PreparedStatement pstmt = connection.prepareStatement("INSERT INTO CMP2_CREDITCARD_ (id_, customer_id, creditOrganization_, expirationDate_, nameOnCard_, number_) VALUES (?, ?, ?, ?, ?, ?)");
            if (paccessor != null) {
                java.lang.Integer id_ =
                    pname.encodeOint();
                if (id_==null) {
                    pstmt.setNull(1, 4);
                } else {
                    pstmt.setObject(1, id_);
                }
                PName customerPN = null;
                if (pClassMapping.customerCodingOnly)
                    customerPN = (PName) paccessor.paGetCustomer(conn);
                else
                    customerPN = pClassMapping.customerPnc.export(conn, paccessor.paGetCustomer(conn));
                java.lang.Integer customer_id =
                    customerPN.encodeOint();
                if (customer_id==null) {
                    pstmt.setNull(2, 4);
                } else {
                    pstmt.setObject(2, customer_id);
                }
                java.lang.String creditOrganization_ =
                    paccessor.paGetCreditOrganization();
                if (creditOrganization_==null) {
                    pstmt.setNull(3, 12);
                } else {
                    pstmt.setString(3, creditOrganization_);
                }
                java.util.Date expirationDate_ =
                    paccessor.paGetExpirationDate();
                if (expirationDate_==null) {
                    pstmt.setNull(4, 91);
                } else {
                    pstmt.setDate(4, (expirationDate_ instanceof java.sql.Date ? (java.sql.Date)expirationDate_ : new java.sql.Date(expirationDate_.getTime())));
                }
                java.lang.String nameOnCard_ =
                    paccessor.paGetNameOnCard();
                if (nameOnCard_==null) {
                    pstmt.setNull(5, 12);
                } else {
                    pstmt.setString(5, nameOnCard_);
                }
                java.lang.String number_ =
                    paccessor.paGetNumber();
                if (number_==null) {
                    pstmt.setNull(6, 12);
                } else {
                    pstmt.setString(6, number_);
                }
            }
            else if (paccessorGen != null) {
                java.lang.Integer id_ =
                    pname.encodeOint();
                if (id_==null) {
                    pstmt.setNull(1, 4);
                } else {
                    pstmt.setObject(1, id_);
                }
                PName customerPN = null;
                if (pClassMapping.customerCodingOnly)
                    customerPN = (PName) paccessorGen.paGetRefField(pClassMapping.customer_NAME, conn);
                else
                    customerPN = pClassMapping.customerPnc.export(conn, paccessorGen.paGetRefField(pClassMapping.customer_NAME, conn));
                java.lang.Integer customer_id =
                    customerPN.encodeOint();
                if (customer_id==null) {
                    pstmt.setNull(2, 4);
                } else {
                    pstmt.setObject(2, customer_id);
                }
                java.lang.String creditOrganization_ =
                    paccessorGen.paGetStringField(pClassMapping.creditOrganization_NAME);
                if (creditOrganization_==null) {
                    pstmt.setNull(3, 12);
                } else {
                    pstmt.setString(3, creditOrganization_);
                }
                java.util.Date expirationDate_ =
                    paccessorGen.paGetDateField(pClassMapping.expirationDate_NAME);
                if (expirationDate_==null) {
                    pstmt.setNull(4, 91);
                } else {
                    pstmt.setDate(4, (expirationDate_ instanceof java.sql.Date ? (java.sql.Date)expirationDate_ : new java.sql.Date(expirationDate_.getTime())));
                }
                java.lang.String nameOnCard_ =
                    paccessorGen.paGetStringField(pClassMapping.nameOnCard_NAME);
                if (nameOnCard_==null) {
                    pstmt.setNull(5, 12);
                } else {
                    pstmt.setString(5, nameOnCard_);
                }
                java.lang.String number_ =
                    paccessorGen.paGetStringField(pClassMapping.number_NAME);
                if (number_==null) {
                    pstmt.setNull(6, 12);
                } else {
                    pstmt.setString(6, number_);
                }
            }
            pstmt.executeUpdate();
			pstmt.close();
		}
		else if (status==PBinding.LIFECYCLE_ACTIVEFORIO) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Update");
             PreparedStatement pstmt = connection.prepareStatement("UPDATE CMP2_CREDITCARD_ SET customer_id = ? , creditOrganization_ = ? , expirationDate_ = ? , nameOnCard_ = ? , number_ = ?  WHERE id_ = ? ");
            if (paccessor != null) {
                java.lang.Integer id_ =
                    pname.encodeOint();
                if (id_==null) {
                    pstmt.setNull(6, 4);
                } else {
                    pstmt.setObject(6, id_);
                }
                PName customerPN = null;
                if (pClassMapping.customerCodingOnly)
                    customerPN = (PName) paccessor.paGetCustomer(conn);
                else
                    customerPN = pClassMapping.customerPnc.export(conn, paccessor.paGetCustomer(conn));
                java.lang.Integer customer_id =
                    customerPN.encodeOint();
                if (customer_id==null) {
                    pstmt.setNull(1, 4);
                } else {
                    pstmt.setObject(1, customer_id);
                }
                java.lang.String creditOrganization_ =
                    paccessor.paGetCreditOrganization();
                if (creditOrganization_==null) {
                    pstmt.setNull(2, 12);
                } else {
                    pstmt.setString(2, creditOrganization_);
                }
                java.util.Date expirationDate_ =
                    paccessor.paGetExpirationDate();
                if (expirationDate_==null) {
                    pstmt.setNull(3, 91);
                } else {
                    pstmt.setDate(3, (expirationDate_ instanceof java.sql.Date ? (java.sql.Date)expirationDate_ : new java.sql.Date(expirationDate_.getTime())));
                }
                java.lang.String nameOnCard_ =
                    paccessor.paGetNameOnCard();
                if (nameOnCard_==null) {
                    pstmt.setNull(4, 12);
                } else {
                    pstmt.setString(4, nameOnCard_);
                }
                java.lang.String number_ =
                    paccessor.paGetNumber();
                if (number_==null) {
                    pstmt.setNull(5, 12);
                } else {
                    pstmt.setString(5, number_);
                }
            }
            else if (paccessorGen != null) {
                java.lang.Integer id_ =
                    pname.encodeOint();
                if (id_==null) {
                    pstmt.setNull(6, 4);
                } else {
                    pstmt.setObject(6, id_);
                }
                PName customerPN = null;
                if (pClassMapping.customerCodingOnly)
                    customerPN = (PName) paccessorGen.paGetRefField(pClassMapping.customer_NAME, conn);
                else
                    customerPN = pClassMapping.customerPnc.export(conn, paccessorGen.paGetRefField(pClassMapping.customer_NAME, conn));
                java.lang.Integer customer_id =
                    customerPN.encodeOint();
                if (customer_id==null) {
                    pstmt.setNull(1, 4);
                } else {
                    pstmt.setObject(1, customer_id);
                }
                java.lang.String creditOrganization_ =
                    paccessorGen.paGetStringField(pClassMapping.creditOrganization_NAME);
                if (creditOrganization_==null) {
                    pstmt.setNull(2, 12);
                } else {
                    pstmt.setString(2, creditOrganization_);
                }
                java.util.Date expirationDate_ =
                    paccessorGen.paGetDateField(pClassMapping.expirationDate_NAME);
                if (expirationDate_==null) {
                    pstmt.setNull(3, 91);
                } else {
                    pstmt.setDate(3, (expirationDate_ instanceof java.sql.Date ? (java.sql.Date)expirationDate_ : new java.sql.Date(expirationDate_.getTime())));
                }
                java.lang.String nameOnCard_ =
                    paccessorGen.paGetStringField(pClassMapping.nameOnCard_NAME);
                if (nameOnCard_==null) {
                    pstmt.setNull(4, 12);
                } else {
                    pstmt.setString(4, nameOnCard_);
                }
                java.lang.String number_ =
                    paccessorGen.paGetStringField(pClassMapping.number_NAME);
                if (number_==null) {
                    pstmt.setNull(5, 12);
                } else {
                    pstmt.setString(5, number_);
                }
            }
            pstmt.executeUpdate();
			pstmt.close();
		}
		}
		catch(java.sql.SQLException se) {
			throw new PExceptionIO(se);
		}
		catch(PException pe) {
			throw new PExceptionIO(pe);
		}

		// Changing the current state
		status = nextState;
	}

	public void read(Object conn, PAccessor pa) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		Cmp2_CreditCardAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof Cmp2_CreditCardAccessor)
			paccessor = (Cmp2_CreditCardAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor");
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized operation");
		if (conn==null)
			throw new PExceptionProtocol("Impossible to read without connection");
		else if (conn instanceof Connection)
			connection = (Connection) conn;
		else if (conn instanceof ConnectionWrapper)
			connection = (Connection) ((ConnectionWrapper) conn).getNonCCIConnection();
		else
			throw new PExceptionProtocol("Impossible to read with this connection type.");
		try {
            Statement stmt = connection.createStatement();
            ResultSet resultSet = null;
			resultSet = stmt.executeQuery("SELECT creditOrganization_, expirationDate_, nameOnCard_, number_, id_, customer_id FROM CMP2_CREDITCARD_ WHERE id_ = " + pname.encodeOint() + " ");
			if(resultSet.next() == false)
				throw new PExceptionNaming("Invalid PName");

			if (paccessor != null) {
				java.lang.String creditOrganization = resultSet.getString(1);
				if (resultSet.wasNull()) creditOrganization = null;
				paccessor.paSetCreditOrganization(
					creditOrganization );
				java.util.Date expirationDate = resultSet.getDate(2);
				if (resultSet.wasNull()) expirationDate = null;
				paccessor.paSetExpirationDate(
					expirationDate );
				java.lang.String nameOnCard = resultSet.getString(3);
				if (resultSet.wasNull()) nameOnCard = null;
				paccessor.paSetNameOnCard(
					nameOnCard );
				java.lang.String number = resultSet.getString(4);
				if (resultSet.wasNull()) number = null;
				paccessor.paSetNumber(
					number );
				java.lang.Integer id = ((java.lang.Integer)resultSet.getObject(5));
				if (resultSet.wasNull()) id = null;
				paccessor.paSetId(
					id );
				java.lang.Integer customer_customer_id = ((java.lang.Integer)resultSet.getObject(6));
				if (resultSet.wasNull()) customer_customer_id = null;

				if (pClassMapping.customerCodingOnly)
					paccessor.paSetCustomer(pClassMapping.customerPnc.decodeOint(customer_customer_id), conn);
				else
					paccessor.paSetCustomer(pClassMapping.customerPnc.decodeOint(customer_customer_id).resolve(conn), conn);
			}
			else if (paccessorGen != null) {
				java.lang.String creditOrganization = resultSet.getString(1);
				if (resultSet.wasNull()) creditOrganization = null;
				paccessorGen.paSetStringField(pClassMapping.creditOrganization_NAME,
					creditOrganization );
				java.util.Date expirationDate = resultSet.getDate(2);
				if (resultSet.wasNull()) expirationDate = null;
				paccessorGen.paSetDateField(pClassMapping.expirationDate_NAME,
					expirationDate );
				java.lang.String nameOnCard = resultSet.getString(3);
				if (resultSet.wasNull()) nameOnCard = null;
				paccessorGen.paSetStringField(pClassMapping.nameOnCard_NAME,
					nameOnCard );
				java.lang.String number = resultSet.getString(4);
				if (resultSet.wasNull()) number = null;
				paccessorGen.paSetStringField(pClassMapping.number_NAME,
					number );
				java.lang.Integer id = ((java.lang.Integer)resultSet.getObject(5));
				if (resultSet.wasNull()) id = null;
				paccessorGen.paSetOintField(pClassMapping.id_NAME,
					id );
				java.lang.Integer customer_customer_id = ((java.lang.Integer)resultSet.getObject(6));
				if (resultSet.wasNull()) customer_customer_id = null;

				if (pClassMapping.customerCodingOnly)
					paccessorGen.paSetRefField(pClassMapping.customer_NAME, pClassMapping.customerPnc.decodeOint(customer_customer_id), conn);
				else
					paccessorGen.paSetRefField(pClassMapping.customer_NAME, pClassMapping.customerPnc.decodeOint(customer_customer_id).resolve(conn), conn);
			}
 			resultSet.close();
			resultSet = null; //garbage the object
			stmt.close();
			stmt = null;

		}
		catch(java.sql.SQLException se) {
			throw new PExceptionIO(se);
		}
		catch(PException pe) {
			throw new PExceptionIO(pe);
		}
		status = nextState;
	}

	public synchronized boolean exist(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_EXIST);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized operation");
		status = nextState;
		if (conn==null)
			throw new PExceptionProtocol("Impossible to check the existence without connection");
		else if (conn instanceof Connection)
			connection = (Connection) conn;
		else if (conn instanceof ConnectionWrapper)
			connection = (Connection) ((ConnectionWrapper) conn).getNonCCIConnection();
		else
			throw new PExceptionProtocol("Impossible to check the existence with this connection type.");
		try {
			Statement stmt = connection.createStatement();
            boolean exist = false;
			exist = stmt.executeQuery("SELECT * FROM CMP2_CREDITCARD_ WHERE id_ = " + pname.encodeOint() + " ").next();
			stmt.close();
			return exist;
		}
		catch(java.sql.SQLException se) {
			throw new PExceptionIO(se);
		}
        catch(PException e) {
			throw new PExceptionIO(e);
		}
	}

	public synchronized PName export(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		return pClassMapping.getPBinder().export(conn, this);
	}

	public synchronized PName export(Object conn, Object hints)
		throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		return pClassMapping.getPBinder().export(conn, this, hints);
	}

	public synchronized void unexport(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		pClassMapping.getPBinder().unexport(conn, this.pname);
	}

	public synchronized void bind(PName pn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		// The Pbinding must be unbinded only if the PName has been already
		// defined
		if (pname!=null)
			pClassMapping.getPBinder().unbind(this);
		pClassMapping.getPBinder().bind(pn, this);
	}

	public synchronized void unbind() throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		pClassMapping.getPBinder().unbind(this);
	}

	public PClassMapping getPClassMapping() {
		return pClassMapping;
	}

	public PName getPName() {
		return pname;
	}

	public byte getStatus() {
		return status;
	}

	// IMPLEMENTATION OF METHODS FROM THE PBindingCtrl INTERFACE

	public void setPName(PName pn) {
		pname = pn;
	}

	public void setStatus(byte s) {
		status = s;
	}

 
}
