/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */
package jorm.rdb.postgres;

// Import clauses required by jorm
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PAccessorGen;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Date;

// Import clauses required by the object Cmp2_Cabin
import jorm.Cmp2_CabinAccessor;
// Imports needed by the rdb mapper
import org.objectweb.transaction.api.ConnectionWrapper;
import java.sql.Blob;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.util.HashMap;

public class Cmp2_CabinBinding
	extends org.objectweb.jonas_ejb.container.JEntitySwitch
	implements PBindingCtrl
 
 {
	/**
	 * The current state of this PBinding object.
	 */
	protected byte status = PBinding.LIFECYCLE_NOTBOUND;

	/**
	 * The PClassMapping linked to this PBinding object.
	 */
	protected Cmp2_CabinMapping pClassMapping = null;

	/**
	 * The PName of this PBinding object.
	 */
	protected PName pname = null;

	// Declare the Connection and the ResultSet used (java.sql)
	private Connection connection = null;
	private ResultSet resultSet = null;

	public Cmp2_CabinBinding() {
		super();
// nothing
	}

	public Cmp2_CabinBinding(PClassMapping pcm) {
		super();
		pClassMapping = (Cmp2_CabinMapping) pcm;
// nothing
	}


	// IMPLEMENTATION OF METHODS FROM THE PBinding INTERFACE

	public void init(PClassMapping pcm) throws PException {
		pClassMapping = (Cmp2_CabinMapping) pcm;
	}

	public synchronized void write(Object conn, PAccessor pa)
		throws PException {
		Cmp2_CabinAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof Cmp2_CabinAccessor)
			paccessor = (Cmp2_CabinAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor");
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_WRITE);
		if (nextState == PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized operation");
		if (conn==null)
			throw new PExceptionProtocol("Impossible to write without connection");
		else if (conn instanceof Connection)
			connection = (Connection) conn;
		else if (conn instanceof ConnectionWrapper)
			connection = (Connection) ((ConnectionWrapper) conn).getNonCCIConnection();
		else
			throw new PExceptionProtocol("Impossible to write with this connection type.");
		try {
			Blob blob;
		if (status == PBinding.LIFECYCLE_DELTOWRITE) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Remove");
            PreparedStatement pstmt = connection.prepareStatement("DELETE FROM CMP2_CABIN_ WHERE id_ = ? ");
 
            java.lang.Integer id_ = pname.encodeOint();
            if (id_==null) {
                pstmt.setNull(1, 4);
            } else {
                pstmt.setObject(1, id_);
            }
            pstmt.executeUpdate();
			pstmt.close();
        }
		else if (status==PBinding.LIFECYCLE_NEWTOWRITE) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Insert");
            PreparedStatement pstmt = connection.prepareStatement("INSERT INTO CMP2_CABIN_ (id_, ship_id, name_, deckLevel_, bedCount_) VALUES (?, ?, ?, ?, ?)");
            if (paccessor != null) {
                java.lang.Integer id_ =
                    pname.encodeOint();
                if (id_==null) {
                    pstmt.setNull(1, 4);
                } else {
                    pstmt.setObject(1, id_);
                }
                PName shipPN = null;
                if (pClassMapping.shipCodingOnly)
                    shipPN = (PName) paccessor.paGetShip(conn);
                else
                    shipPN = pClassMapping.shipPnc.export(conn, paccessor.paGetShip(conn));
                java.lang.Integer ship_id =
                    shipPN.encodeOint();
                if (ship_id==null) {
                    pstmt.setNull(2, 4);
                } else {
                    pstmt.setObject(2, ship_id);
                }
                java.lang.String name_ =
                    paccessor.paGetName();
                if (name_==null) {
                    pstmt.setNull(3, 12);
                } else {
                    pstmt.setString(3, name_);
                }
                pstmt.setInt(4, paccessor.paGetDeckLevel());
                pstmt.setInt(5, paccessor.paGetBedCount());
            }
            else if (paccessorGen != null) {
                java.lang.Integer id_ =
                    pname.encodeOint();
                if (id_==null) {
                    pstmt.setNull(1, 4);
                } else {
                    pstmt.setObject(1, id_);
                }
                PName shipPN = null;
                if (pClassMapping.shipCodingOnly)
                    shipPN = (PName) paccessorGen.paGetRefField(pClassMapping.ship_NAME, conn);
                else
                    shipPN = pClassMapping.shipPnc.export(conn, paccessorGen.paGetRefField(pClassMapping.ship_NAME, conn));
                java.lang.Integer ship_id =
                    shipPN.encodeOint();
                if (ship_id==null) {
                    pstmt.setNull(2, 4);
                } else {
                    pstmt.setObject(2, ship_id);
                }
                java.lang.String name_ =
                    paccessorGen.paGetStringField(pClassMapping.name_NAME);
                if (name_==null) {
                    pstmt.setNull(3, 12);
                } else {
                    pstmt.setString(3, name_);
                }
                pstmt.setInt(4, paccessorGen.paGetIntField(pClassMapping.deckLevel_NAME));
                pstmt.setInt(5, paccessorGen.paGetIntField(pClassMapping.bedCount_NAME));
            }
            pstmt.executeUpdate();
			pstmt.close();
		}
		else if (status==PBinding.LIFECYCLE_ACTIVEFORIO) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Update");
             PreparedStatement pstmt = connection.prepareStatement("UPDATE CMP2_CABIN_ SET ship_id = ? , name_ = ? , deckLevel_ = ? , bedCount_ = ?  WHERE id_ = ? ");
            if (paccessor != null) {
                java.lang.Integer id_ =
                    pname.encodeOint();
                if (id_==null) {
                    pstmt.setNull(5, 4);
                } else {
                    pstmt.setObject(5, id_);
                }
                PName shipPN = null;
                if (pClassMapping.shipCodingOnly)
                    shipPN = (PName) paccessor.paGetShip(conn);
                else
                    shipPN = pClassMapping.shipPnc.export(conn, paccessor.paGetShip(conn));
                java.lang.Integer ship_id =
                    shipPN.encodeOint();
                if (ship_id==null) {
                    pstmt.setNull(1, 4);
                } else {
                    pstmt.setObject(1, ship_id);
                }
                java.lang.String name_ =
                    paccessor.paGetName();
                if (name_==null) {
                    pstmt.setNull(2, 12);
                } else {
                    pstmt.setString(2, name_);
                }
                pstmt.setInt(3, paccessor.paGetDeckLevel());
                pstmt.setInt(4, paccessor.paGetBedCount());
            }
            else if (paccessorGen != null) {
                java.lang.Integer id_ =
                    pname.encodeOint();
                if (id_==null) {
                    pstmt.setNull(5, 4);
                } else {
                    pstmt.setObject(5, id_);
                }
                PName shipPN = null;
                if (pClassMapping.shipCodingOnly)
                    shipPN = (PName) paccessorGen.paGetRefField(pClassMapping.ship_NAME, conn);
                else
                    shipPN = pClassMapping.shipPnc.export(conn, paccessorGen.paGetRefField(pClassMapping.ship_NAME, conn));
                java.lang.Integer ship_id =
                    shipPN.encodeOint();
                if (ship_id==null) {
                    pstmt.setNull(1, 4);
                } else {
                    pstmt.setObject(1, ship_id);
                }
                java.lang.String name_ =
                    paccessorGen.paGetStringField(pClassMapping.name_NAME);
                if (name_==null) {
                    pstmt.setNull(2, 12);
                } else {
                    pstmt.setString(2, name_);
                }
                pstmt.setInt(3, paccessorGen.paGetIntField(pClassMapping.deckLevel_NAME));
                pstmt.setInt(4, paccessorGen.paGetIntField(pClassMapping.bedCount_NAME));
            }
            pstmt.executeUpdate();
			pstmt.close();
		}
		}
		catch(java.sql.SQLException se) {
			throw new PExceptionIO(se);
		}
		catch(PException pe) {
			throw new PExceptionIO(pe);
		}

		// Changing the current state
		status = nextState;
	}

	public void read(Object conn, PAccessor pa) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		Cmp2_CabinAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof Cmp2_CabinAccessor)
			paccessor = (Cmp2_CabinAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor");
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized operation");
		if (conn==null)
			throw new PExceptionProtocol("Impossible to read without connection");
		else if (conn instanceof Connection)
			connection = (Connection) conn;
		else if (conn instanceof ConnectionWrapper)
			connection = (Connection) ((ConnectionWrapper) conn).getNonCCIConnection();
		else
			throw new PExceptionProtocol("Impossible to read with this connection type.");
		try {
            Statement stmt = connection.createStatement();
            ResultSet resultSet = null;
			resultSet = stmt.executeQuery("SELECT name_, deckLevel_, bedCount_, id_, ship_id FROM CMP2_CABIN_ WHERE id_ = " + pname.encodeOint() + " ");
			if(resultSet.next() == false)
				throw new PExceptionNaming("Invalid PName");

			if (paccessor != null) {
				java.lang.String name = resultSet.getString(1);
				if (resultSet.wasNull()) name = null;
				paccessor.paSetName(
					name );
				paccessor.paSetDeckLevel(
					resultSet.getInt(2) );
				paccessor.paSetBedCount(
					resultSet.getInt(3) );
				java.lang.Integer id = ((java.lang.Integer)resultSet.getObject(4));
				if (resultSet.wasNull()) id = null;
				paccessor.paSetId(
					id );
				java.lang.Integer ship_ship_id = ((java.lang.Integer)resultSet.getObject(5));
				if (resultSet.wasNull()) ship_ship_id = null;

				if (pClassMapping.shipCodingOnly)
					paccessor.paSetShip(pClassMapping.shipPnc.decodeOint(ship_ship_id), conn);
				else
					paccessor.paSetShip(pClassMapping.shipPnc.decodeOint(ship_ship_id).resolve(conn), conn);
				java.lang.Integer jonasCMRCabin_Reservation_id = ((java.lang.Integer)resultSet.getObject(4));
				if (resultSet.wasNull()) jonasCMRCabin_Reservation_id = null;

				if (pClassMapping.jonasCMRCabin_ReservationCodingOnly)
					paccessor.paSetJonasCMRCabin_Reservation(pClassMapping.jonasCMRCabin_ReservationPnc.decodeOint(jonasCMRCabin_Reservation_id), conn);
				else
					paccessor.paSetJonasCMRCabin_Reservation(pClassMapping.jonasCMRCabin_ReservationPnc.decodeOint(jonasCMRCabin_Reservation_id).resolve(conn), conn);
			}
			else if (paccessorGen != null) {
				java.lang.String name = resultSet.getString(1);
				if (resultSet.wasNull()) name = null;
				paccessorGen.paSetStringField(pClassMapping.name_NAME,
					name );
				paccessorGen.paSetIntField(pClassMapping.deckLevel_NAME,
					resultSet.getInt(2) );
				paccessorGen.paSetIntField(pClassMapping.bedCount_NAME,
					resultSet.getInt(3) );
				java.lang.Integer id = ((java.lang.Integer)resultSet.getObject(4));
				if (resultSet.wasNull()) id = null;
				paccessorGen.paSetOintField(pClassMapping.id_NAME,
					id );
				java.lang.Integer ship_ship_id = ((java.lang.Integer)resultSet.getObject(5));
				if (resultSet.wasNull()) ship_ship_id = null;

				if (pClassMapping.shipCodingOnly)
					paccessorGen.paSetRefField(pClassMapping.ship_NAME, pClassMapping.shipPnc.decodeOint(ship_ship_id), conn);
				else
					paccessorGen.paSetRefField(pClassMapping.ship_NAME, pClassMapping.shipPnc.decodeOint(ship_ship_id).resolve(conn), conn);
				java.lang.Integer jonasCMRCabin_Reservation_id = ((java.lang.Integer)resultSet.getObject(4));
				if (resultSet.wasNull()) jonasCMRCabin_Reservation_id = null;

				if (pClassMapping.jonasCMRCabin_ReservationCodingOnly)
					paccessorGen.paSetRefField(pClassMapping.jonasCMRCabin_Reservation_NAME, pClassMapping.jonasCMRCabin_ReservationPnc.decodeOint(jonasCMRCabin_Reservation_id), conn);
				else
					paccessorGen.paSetRefField(pClassMapping.jonasCMRCabin_Reservation_NAME, pClassMapping.jonasCMRCabin_ReservationPnc.decodeOint(jonasCMRCabin_Reservation_id).resolve(conn), conn);
			}
 			resultSet.close();
			resultSet = null; //garbage the object
			stmt.close();
			stmt = null;

		}
		catch(java.sql.SQLException se) {
			throw new PExceptionIO(se);
		}
		catch(PException pe) {
			throw new PExceptionIO(pe);
		}
		status = nextState;
	}

	public synchronized boolean exist(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_EXIST);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized operation");
		status = nextState;
		if (conn==null)
			throw new PExceptionProtocol("Impossible to check the existence without connection");
		else if (conn instanceof Connection)
			connection = (Connection) conn;
		else if (conn instanceof ConnectionWrapper)
			connection = (Connection) ((ConnectionWrapper) conn).getNonCCIConnection();
		else
			throw new PExceptionProtocol("Impossible to check the existence with this connection type.");
		try {
			Statement stmt = connection.createStatement();
            boolean exist = false;
			exist = stmt.executeQuery("SELECT * FROM CMP2_CABIN_ WHERE id_ = " + pname.encodeOint() + " ").next();
			stmt.close();
			return exist;
		}
		catch(java.sql.SQLException se) {
			throw new PExceptionIO(se);
		}
        catch(PException e) {
			throw new PExceptionIO(e);
		}
	}

	public synchronized PName export(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		return pClassMapping.getPBinder().export(conn, this);
	}

	public synchronized PName export(Object conn, Object hints)
		throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		return pClassMapping.getPBinder().export(conn, this, hints);
	}

	public synchronized void unexport(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		pClassMapping.getPBinder().unexport(conn, this.pname);
	}

	public synchronized void bind(PName pn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		// The Pbinding must be unbinded only if the PName has been already
		// defined
		if (pname!=null)
			pClassMapping.getPBinder().unbind(this);
		pClassMapping.getPBinder().bind(pn, this);
	}

	public synchronized void unbind() throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		pClassMapping.getPBinder().unbind(this);
	}

	public PClassMapping getPClassMapping() {
		return pClassMapping;
	}

	public PName getPName() {
		return pname;
	}

	public byte getStatus() {
		return status;
	}

	// IMPLEMENTATION OF METHODS FROM THE PBindingCtrl INTERFACE

	public void setPName(PName pn) {
		pname = pn;
	}

	public void setStatus(byte s) {
		status = s;
	}

 
}
